package de.subway_surfers.vpr_app;

import Logik.Mahlzeit;
import Logik.Tagesplan;
import RestAPISchnittstelle.RestApiClient;
import javafx.beans.value.ChangeListener;
import javafx.beans.value.ObservableValue;
import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import javafx.event.ActionEvent;
import javafx.fxml.FXML;
import javafx.geometry.Pos;
import javafx.scene.Node;
import javafx.scene.control.Button;
import javafx.scene.control.Control;
import javafx.scene.control.Label;
import javafx.scene.layout.ColumnConstraints;
import javafx.scene.layout.GridPane;
import javafx.scene.layout.Priority;
import javafx.scene.layout.RowConstraints;
import javafx.stage.Modality;
import javafx.stage.Stage;

import java.net.http.WebSocket;
import java.text.DateFormat;
import java.time.LocalTime;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.Date;

import static java.time.LocalTime.now;


public class EssensverwaltungMitarbeiterView {

	public Label dateLabel;
	@FXML
	private GridPane tagesplan;

	private Tagesplan t;

	private String date;

	private String day;

	private String month;

	private String year;

	private ChangeListener listener;

	public void initialize(){

/*
		(obs,oldValue,newValue) -> {
			final int zeile = 1;
			for (Node n : tagesplan.getChildren()){
				if(n instanceof Control  && GridPane.getRowIndex(n) == zeile){
					((Control) n).setPrefHeight(newValue.floatValue());
				}
			}
		}
		*/

		DateFormat dateFormat = DateFormat.getDateInstance();
		date = dateFormat.format(new Date());
		date = date.replace('.', '-');
		day = date.split("-")[0];
		month = date.split("-")[1];
		year = date.split("-")[2];
		date = String.format("%s-%s-%s", year, month, day);

		dateLabel.setText(String.format("%s.%s.%s", day, month, year));

		initGrid();
		initGerichte();

	}

	public void onAbmelden(ActionEvent actionEvent) {
		VerwaltungApplication.sceneWechseln("login-view.fxml");
	}

	public void onFilter(ActionEvent actionEvent) {
		Stage stage = new Stage();
		VerwaltungApplication.sceneWechseln(stage, 450, 400, "inhaltsstoffe_filtern-view.fxml");

	}

	public void onHinzufuegen(ActionEvent actionEvent) {
		Stage gerichterstellung = new Stage();

		//solange das neu geöffnete Fenster offen ist, wird das Hauptfenster gesperrt
		gerichterstellung.initModality(Modality.APPLICATION_MODAL);

		VerwaltungApplication.sceneWechseln(gerichterstellung, 400, 530, "gerichterstellung_mitarbeiter-view.fxml");

		gerichterstellung.minWidthProperty().set(420);
		gerichterstellung.minHeightProperty().set(600);
	}

	public void onZurueck(ActionEvent actionEvent) {
		VerwaltungApplication.sceneWechseln("hauptmenue_mitarbeiter-view.fxml");
	}

	/**
	 * Methode bei Klick auf Pfeil links, ändert das Datum und gibt neue Gerichte auf die GUI aus.
	 * @param actionEvent
	 */
	public void pfeilLinks(ActionEvent actionEvent) {

		tagesplan.getChildren().clear();

		day = String.valueOf(Integer.parseInt(day)-1);
		date = String.format("%s-%s-%s", year, month, day);

		dateLabel.setText(String.format("%s.%s.%s", day, month, year));

		initGerichte();

	}

	/**
	 * Methode bei Klick auf Pfeil rechts, ändert das Datum und gibt neue Gerichte auf die GUI aus.
	 * @param actionEvent
	 */
	public void pfeilRechts(ActionEvent actionEvent) {

		tagesplan.getChildren().clear();

		day = String.valueOf(Integer.parseInt(day)+1);
		date = String.format("%s-%s-%s", year, month, day);

		dateLabel.setText(String.format("%s.%s.%s", day, month, year));

		initGerichte();

	}

	/**
	 * Initialisiert alle Gerichte des aktuellen Datums auf der Seite.
	 */
	private void initGerichte(){

		tagesplan.getStyleClass().clear();

		t = new RestApiClient().getGerichteOnTag(date);

		if(!t.getGerichte().isEmpty()) {
			for (int i = 0; i < t.getGerichte().size(); i++) {
				Label name = new Label();
				tagesplan.add(name, i, 0);
				Label beschreibung = new Label();
				beschreibung.setWrapText(true);
				tagesplan.add(beschreibung, i, 1);
				Button loeschen = new Button("Löschen");
				tagesplan.add(loeschen, i ,2);
				loeschen.setOnAction(this::loeschenButtonKlick);

				name.setText(t.getGerichte().get(i).getName());
				beschreibung.setText(t.getGerichte().get(i).getBeschreibung());
			}
		}
		else {
			Label label = new Label();

			label.setText("Für diesen Tag sind keine Gerichte eingetragen");
			tagesplan.add(label, 0, 0);
		}

		for (Node n : tagesplan.getChildren()) {
			if (n instanceof Control) {
				((Control) n).setPrefWidth(tagesplan.getWidth() / tagesplan.getColumnCount());
			}
		}

		for (Node n : tagesplan.getChildren()){
			if(n instanceof Control  && GridPane.getRowIndex(n) == 1){
				((Control) n).setPrefHeight(tagesplan.getHeight());
			}
		}

		tagesplan.getStyleClass().add("essensuebersicht_gridlines");
	}

	public void initGrid(){
		for (int i = 0; i < tagesplan.getColumnCount(); i++) {
			ColumnConstraints cc = new ColumnConstraints();
			cc.setHgrow(Priority.ALWAYS);
			cc.setFillWidth(true);
			tagesplan.getColumnConstraints().add(cc);
		}

		tagesplan.heightProperty().addListener((obs,oldValue,newValue) -> {
			final int zeile = 1;
			for (Node n : tagesplan.getChildren()){
				if(n instanceof Control  && GridPane.getRowIndex(n) == zeile){
					((Control) n).setPrefHeight(newValue.floatValue());
				}
			}
		});

		VerwaltungApplication.responsiveBreiteGrid(tagesplan);
	}

	private void loeschenButtonKlick(ActionEvent a){
		Button btn = (Button) a.getSource();
		int col = GridPane.getColumnIndex(btn);
		t.getGerichte().remove(col);
		initGerichte();
	}



}
